@@ -20,6 +20,7 @@
 #include "nsCocoaUtils.h"
 #include "gfxQuartzSurface.h"
 #include "GLContextTypes.h"
+#include "mozilla/DataMutex.h"
 #include "mozilla/Mutex.h"
 #include "nsRegion.h"
 #include "mozilla/MouseEvents.h"
@@ -29,6 +30,7 @@
 #include "nsString.h"
 #include "nsIDragService.h"
 #include "ViewRegion.h"
+#include "CFTypeRefPtr.h"
 
 #import <Carbon/Carbon.h>
 #import <Cocoa/Cocoa.h>
@@ -50,9 +52,10 @@ class VibrancyManager;
 namespace layers {
 class GLManager;
 class IAPZCTreeManager;
+class NativeLayerRootCA;
+class NativeLayerCA;
 }  // namespace layers
 namespace widget {
-class RectTextureImage;
 class WidgetRenderingContext;
 }  // namespace widget
 }  // namespace mozilla
@@ -73,27 +76,6 @@ class WidgetRenderingContext;
 
 @interface NSView (Undocumented)
 
-// Draws the title string of a window.
-// Present on NSThemeFrame since at least 10.6.
-// _drawTitleBar is somewhat complex, and has changed over the years
-// since OS X 10.6.  But in that time it's never done anything that
-// would break when called outside of -[NSView drawRect:] (which we
-// sometimes do), or whose output can't be redirected to a
-// CGContextRef object (which we also sometimes do).  This is likely
-// to remain true for the indefinite future.  However we should
-// check _drawTitleBar in each new major version of OS X.  For more
-// information see bug 877767.
-- (void)_drawTitleBar:(NSRect)aRect;
-
-// Returns an NSRect that is the bounding box for all an NSView's dirty
-// rectangles (ones that need to be redrawn).  The full list of dirty
-// rectangles can be obtained by calling -[NSView _dirtyRegion] and then
-// calling -[NSRegion getRects:count:] on what it returns.  Both these
-// methods have been present in the same form since at least OS X 10.5.
-// Unlike -[NSView getRectsBeingDrawn:count:], these methods can be called
-// outside a call to -[NSView drawRect:].
-- (NSRect)_dirtyRect;
-
 // Undocumented method of one or more of NSFrameView's subclasses.  Called
 // when one or more of the titlebar buttons needs to be repositioned, to
 // disappear, or to reappear (say if the window's style changes).  If
@@ -155,9 +137,8 @@ class WidgetRenderingContext;
   // to send its pair event first, in case we didn't yet for any reason.
   BOOL mExpectingWheelStop;
 
-  // Set to YES when our GL surface has been updated and we need to call
-  // updateGLContext before we composite.
-  BOOL mNeedsGLUpdate;
+  // Whether we're inside updateRootCALayer at the moment.
+  BOOL mIsUpdatingLayer;
 
   // Holds our drag service across multiple drag calls. The reference to the
   // service is obtained when the mouse enters the view and is released when
@@ -166,8 +147,6 @@ class WidgetRenderingContext;
   // when handling |draggingUpdated:| messages.
   nsIDragService* mDragService;
 
-  NSOpenGLContext* mGLContext;
-
   // Gestures support
   //
   // mGestureState is used to detect when Cocoa has called both
@@ -175,10 +154,6 @@ class WidgetRenderingContext;
   // beginGestureWithEvent and endGestureWithEvent sequence. We
   // discard the spurious gesture event so as not to confuse Gecko.
   //
-  // mCumulativeMagnification keeps track of the total amount of
-  // magnification peformed during a magnify gesture so that we can
-  // send that value with the final MozMagnifyGesture event.
-  //
   // mCumulativeRotation keeps track of the total amount of rotation
   // performed during a rotate gesture so we can send that value with
   // the final MozRotateGesture event.
@@ -188,7 +163,6 @@ class WidgetRenderingContext;
     eGestureState_MagnifyGesture,
     eGestureState_RotateGesture
   } mGestureState;
-  float mCumulativeMagnification;
   float mCumulativeRotation;
 
 #ifdef __LP64__
@@ -199,17 +173,13 @@ class WidgetRenderingContext;
   // Whether this uses off-main-thread compositing.
   BOOL mUsingOMTCompositor;
 
-  // The mask image that's used when painting into the titlebar using basic
-  // CGContext painting (i.e. non-accelerated).
-  CGImageRef mTopLeftCornerMask;
-
   // Subviews of self, which act as container views for vibrancy views and
   // non-draggable views.
   NSView* mVibrancyViewsContainer;      // [STRONG]
   NSView* mNonDraggableViewsContainer;  // [STRONG]
 
-  // The view that does our drawing. This is a subview of self so that it can
-  // be ordered on top of mVibrancyViewsContainer.
+  // The layer-backed view that hosts our drawing. Always non-null.
+  // This is a subview of self so that it can be ordered on top of mVibrancyViewsContainer.
   PixelHostingView* mPixelHostingView;
 
   // Last pressure stage by trackpad's force click
@@ -234,11 +204,13 @@ class WidgetRenderingContext;
                             enter:(BOOL)aEnter
                          exitFrom:(mozilla::WidgetMouseEvent::ExitFrom)aExitFrom;
 
-- (void)updateGLContext;
-- (void)_surfaceNeedsUpdate:(NSNotification*)notification;
-
-- (bool)preRender:(NSOpenGLContext*)aGLContext;
-- (void)postRender:(NSOpenGLContext*)aGLContext;
+// Call this during operations that will likely trigger a main thread
+// CoreAnimation paint of the window, during which Gecko should do its own
+// painting and present the results atomically with that main thread transaction.
+// This method will suspend off-thread window updates so that the upcoming paint
+// can be atomic, and mark the layer as needing display so that
+// HandleMainThreadCATransaction gets called and Gecko gets a chance to paint.
+- (void)ensureNextCompositeIsAtomicWithMainThreadPaint;
 
 - (NSView*)vibrancyViewsContainer;
 - (NSView*)nonDraggableViewsContainer;
@@ -249,8 +221,6 @@ class WidgetRenderingContext;
 - (void)viewWillStartLiveResize;
 - (void)viewDidEndLiveResize;
 
-- (NSColor*)vibrancyFillColorForThemeGeometryType:(nsITheme::ThemeGeometryType)aThemeGeometryType;
-
 /*
  * Gestures support
  *
@@ -313,9 +283,9 @@ class nsChildView final : public nsBaseWidget {
   nsChildView();
 
   // nsIWidget interface
-  virtual MOZ_MUST_USE nsresult Create(nsIWidget* aParent, nsNativeWidget aNativeParent,
-                                       const LayoutDeviceIntRect& aRect,
-                                       nsWidgetInitData* aInitData = nullptr) override;
+  [[nodiscard]] virtual nsresult Create(nsIWidget* aParent, nsNativeWidget aNativeParent,
+                                        const LayoutDeviceIntRect& aRect,
+                                        nsWidgetInitData* aInitData = nullptr) override;
 
   virtual void Destroy() override;
 
@@ -332,7 +302,7 @@ class nsChildView final : public nsBaseWidget {
 
   virtual void Enable(bool aState) override;
   virtual bool IsEnabled() const override;
-  virtual nsresult SetFocus(bool aRaise) override;
+  virtual void SetFocus(Raise, mozilla::dom::CallerType aCallerType) override;
   virtual LayoutDeviceIntRect GetBounds() override;
   virtual LayoutDeviceIntRect GetClientBounds() override;
   virtual LayoutDeviceIntRect GetScreenBounds() override;
@@ -365,6 +335,7 @@ class nsChildView final : public nsBaseWidget {
   virtual int32_t RoundsWidgetCoordinatesTo() override;
 
   virtual void Invalidate(const LayoutDeviceIntRect& aRect) override;
+  void EnsureContentLayerForMainThreadPainting();
 
   virtual void* GetNativeData(uint32_t aDataType) override;
   virtual nsresult ConfigureChildren(const nsTArray<Configuration>& aConfigurations) override;
@@ -384,7 +355,7 @@ class nsChildView final : public nsBaseWidget {
 
   virtual nsresult SetTitle(const nsAString& title) override;
 
-  virtual MOZ_MUST_USE nsresult GetAttention(int32_t aCycleCount) override;
+  [[nodiscard]] virtual nsresult GetAttention(int32_t aCycleCount) override;
 
   virtual bool HasPendingInputEvent() override;
 
@@ -392,14 +363,15 @@ class nsChildView final : public nsBaseWidget {
   virtual void PostHandleKeyEvent(mozilla::WidgetKeyboardEvent* aEvent) override;
   virtual nsresult ActivateNativeMenuItemAt(const nsAString& indexString) override;
   virtual nsresult ForceUpdateNativeMenuAt(const nsAString& indexString) override;
-  virtual MOZ_MUST_USE nsresult GetSelectionAsPlaintext(nsAString& aResult) override;
+  [[nodiscard]] virtual nsresult GetSelectionAsPlaintext(nsAString& aResult) override;
 
   virtual void SetInputContext(const InputContext& aContext,
                                const InputContextAction& aAction) override;
   virtual InputContext GetInputContext() override;
   virtual TextEventDispatcherListener* GetNativeTextEventDispatcherListener() override;
-  virtual MOZ_MUST_USE nsresult AttachNativeKeyEvent(mozilla::WidgetKeyboardEvent& aEvent) override;
-  virtual void GetEditCommands(NativeKeyBindingsType aType,
+  [[nodiscard]] virtual nsresult AttachNativeKeyEvent(
+      mozilla::WidgetKeyboardEvent& aEvent) override;
+  virtual bool GetEditCommands(NativeKeyBindingsType aType,
                                const mozilla::WidgetKeyboardEvent& aEvent,
                                nsTArray<mozilla::CommandInt>& aCommands) override;
   void GetEditCommandsRemapped(NativeKeyBindingsType aType,
@@ -409,6 +381,7 @@ class nsChildView final : public nsBaseWidget {
 
   virtual nsTransparencyMode GetTransparencyMode() override;
   virtual void SetTransparencyMode(nsTransparencyMode aMode) override;
+  virtual void SuppressAnimation(bool aSuppress) override;
 
   virtual nsresult SynthesizeNativeKeyEvent(int32_t aNativeKeyboardLayout, int32_t aNativeKeyCode,
                                             uint32_t aModifierFlags, const nsAString& aCharacters,
@@ -442,25 +415,19 @@ class nsChildView final : public nsBaseWidget {
   bool PaintWindow(LayoutDeviceIntRegion aRegion);
   bool PaintWindowInDrawTarget(mozilla::gfx::DrawTarget* aDT, const LayoutDeviceIntRegion& aRegion,
                                const mozilla::gfx::IntSize& aSurfaceSize);
-  bool PaintWindowInContext(CGContextRef aContext, const LayoutDeviceIntRegion& aRegion,
-                            mozilla::gfx::IntSize aSurfaceSize);
+
+  void PaintWindowInContentLayer();
+  void HandleMainThreadCATransaction();
 
 #ifdef ACCESSIBILITY
   already_AddRefed<mozilla::a11y::Accessible> GetDocumentAccessible();
 #endif
 
   virtual void CreateCompositor() override;
-  virtual void PrepareWindowEffects() override;
-  virtual void CleanupWindowEffects() override;
-
-  virtual void AddWindowOverlayWebRenderCommands(
-      mozilla::layers::WebRenderBridgeChild* aWrBridge, mozilla::wr::DisplayListBuilder& aBuilder,
-      mozilla::wr::IpcResourceUpdateQueue& aResourceUpdates) override;
 
   virtual bool PreRender(mozilla::widget::WidgetRenderingContext* aContext) override;
   virtual void PostRender(mozilla::widget::WidgetRenderingContext* aContext) override;
-  virtual void DrawWindowOverlay(mozilla::widget::WidgetRenderingContext* aManager,
-                                 LayoutDeviceIntRect aRect) override;
+  virtual RefPtr<mozilla::layers::NativeLayerRoot> GetNativeLayerRoot() override;
 
   virtual void UpdateThemeGeometries(const nsTArray<ThemeGeometry>& aThemeGeometries) override;
 
@@ -482,16 +449,12 @@ class nsChildView final : public nsBaseWidget {
 
   NSView<mozView>* GetEditorView();
 
-  nsCocoaWindow* GetXULWindowWidget() const;
+  nsCocoaWindow* GetAppWindowWidget() const;
 
   virtual void ReparentNativeWidget(nsIWidget* aNewParent) override;
 
   mozilla::widget::TextInputHandler* GetTextInputHandler() { return mTextInputHandler; }
 
-  NSColor* VibrancyFillColorForThemeGeometryType(nsITheme::ThemeGeometryType aThemeGeometryType);
-  NSColor* VibrancyFontSmoothingBackgroundColorForThemeGeometryType(
-      nsITheme::ThemeGeometryType aThemeGeometryType);
-
   // unit conversion convenience functions
   int32_t CocoaPointsToDevPixels(CGFloat aPts) const {
     return nsCocoaUtils::CocoaPointsToDevPixels(aPts, BackingScaleFactor());
@@ -512,15 +475,9 @@ class nsChildView final : public nsBaseWidget {
     return nsCocoaUtils::DevPixelsToCocoaPoints(aRect, BackingScaleFactor());
   }
 
-  already_AddRefed<mozilla::gfx::DrawTarget> StartRemoteDrawingInRegion(
-      LayoutDeviceIntRegion& aInvalidRegion, mozilla::layers::BufferMode* aBufferMode) override;
-  void EndRemoteDrawing() override;
-  void CleanupRemoteDrawing() override;
-  bool InitCompositor(mozilla::layers::Compositor* aCompositor) override;
-
-  virtual MOZ_MUST_USE nsresult StartPluginIME(const mozilla::WidgetKeyboardEvent& aKeyboardEvent,
-                                               int32_t aPanelX, int32_t aPanelY,
-                                               nsString& aCommitted) override;
+  [[nodiscard]] virtual nsresult StartPluginIME(const mozilla::WidgetKeyboardEvent& aKeyboardEvent,
+                                                int32_t aPanelX, int32_t aPanelY,
+                                                nsString& aCommitted) override;
 
   virtual void SetPluginFocused(bool& aFocused) override;
 
@@ -528,13 +485,29 @@ class nsChildView final : public nsBaseWidget {
 
   virtual LayoutDeviceIntPoint GetClientOffset() override;
 
+  virtual LayoutDeviceIntRegion GetOpaqueWidgetRegion() override;
+
   void DispatchAPZWheelInputEvent(mozilla::InputData& aEvent, bool aCanTriggerSwipe);
   nsEventStatus DispatchAPZInputEvent(mozilla::InputData& aEvent);
 
   void SwipeFinished();
 
-  nsresult SetPrefersReducedMotionOverrideForTest(bool aValue) override;
-  nsresult ResetPrefersReducedMotionOverrideForTest() override;
+  // Called when the main thread enters a phase during which visual changes
+  // are imminent and any layer updates on the compositor thread would interfere
+  // with visual atomicity.
+  // "Async" CATransactions are CATransactions which happen on a thread that's
+  // not the main thread.
+  void SuspendAsyncCATransactions();
+
+  // Called when we know that the current main thread paint will be completed once
+  // the main thread goes back to the event loop.
+  void MaybeScheduleUnsuspendAsyncCATransactions();
+
+  // Called from the runnable dispatched by MaybeScheduleUnsuspendAsyncCATransactions().
+  // At this point we know that the main thread is done handling the visual change
+  // (such as a window resize) and we can start modifying CALayers from the
+  // compositor thread again.
+  void UnsuspendAsyncCATransactions();
 
  protected:
   virtual ~nsChildView();
@@ -551,22 +524,11 @@ class nsChildView final : public nsBaseWidget {
   void ConfigureAPZCTreeManager() override;
   void ConfigureAPZControllerThread() override;
 
-  void DoRemoteComposition(const LayoutDeviceIntRect& aRenderRect);
-
-  // Overlay drawing functions for OpenGL drawing
-  void DrawWindowOverlay(mozilla::layers::GLManager* aManager, LayoutDeviceIntRect aRect);
-  void MaybeDrawRoundedCorners(mozilla::layers::GLManager* aManager,
-                               const LayoutDeviceIntRect& aRect);
-  void MaybeDrawTitlebar(mozilla::layers::GLManager* aManager);
-
-  // Redraw the contents of mTitlebarCGContext on the main thread, as
-  // determined by mDirtyTitlebarRegion.
-  void UpdateTitlebarCGContext();
-
-  LayoutDeviceIntRect RectContainingTitlebarControls();
   void UpdateVibrancy(const nsTArray<ThemeGeometry>& aThemeGeometries);
   mozilla::VibrancyManager& EnsureVibrancyManager();
 
+  void UpdateInternalOpaqueRegion();
+
   nsIWidget* GetWidgetForListenerEvents();
 
   struct SwipeInfo {
@@ -596,35 +558,6 @@ class nsChildView final : public nsBaseWidget {
   // progress on the compositor thread.
   mozilla::Mutex mViewTearDownLock;
 
-  mozilla::Mutex mEffectsLock;
-
-  // May be accessed from any thread, protected
-  // by mEffectsLock.
-  bool mHasRoundedBottomCorners;
-  int mDevPixelCornerRadius;
-  bool mIsCoveringTitlebar;
-  bool mIsFullscreen;
-  bool mIsOpaque;
-  LayoutDeviceIntRect mTitlebarRect;
-
-  // The area of mTitlebarCGContext that needs to be redrawn during the next
-  // transaction. Accessed from any thread, protected by mEffectsLock.
-  LayoutDeviceIntRegion mUpdatedTitlebarRegion;
-  CGContextRef mTitlebarCGContext;
-
-  // Compositor thread only
-  mozilla::UniquePtr<mozilla::widget::RectTextureImage> mCornerMaskImage;
-  mozilla::UniquePtr<mozilla::widget::RectTextureImage> mTitlebarImage;
-  mozilla::UniquePtr<mozilla::widget::RectTextureImage> mBasicCompositorImage;
-
-  // Main thread + webrender only
-  mozilla::Maybe<mozilla::wr::ImageKey> mTitlebarImageKey;
-  mozilla::gfx::IntSize mTitlebarImageSize;
-
-  // The area of mTitlebarCGContext that has changed and needs to be
-  // uploaded to to mTitlebarImage. Main thread only.
-  nsIntRegion mDirtyTitlebarRegion;
-
   mozilla::ViewRegion mNonDraggableRegion;
 
   // Cached value of [mView backingScaleFactor], to avoid sending two obj-c
@@ -639,16 +572,26 @@ class nsChildView final : public nsBaseWidget {
 
   bool mPluginFocused;
 
-  // Used in OMTC BasicLayers mode. Presents the BasicCompositor result
-  // surface to the screen using an OpenGL context.
-  mozilla::UniquePtr<GLPresenter> mGLPresenter;
+  RefPtr<mozilla::layers::NativeLayerRootCA> mNativeLayerRoot;
+
+  // In BasicLayers mode, this is the CoreAnimation layer that contains the
+  // rendering from Gecko. It is a sublayer of mNativeLayerRoot's underlying
+  // wrapper layer.
+  // Lazily created by EnsureContentLayerForMainThreadPainting().
+  RefPtr<mozilla::layers::NativeLayerCA> mContentLayer;
+  RefPtr<mozilla::layers::SurfacePoolHandle> mPoolHandle;
+
+  // In BasicLayers mode, this is the invalid region of mContentLayer.
+  LayoutDeviceIntRegion mContentLayerInvalidRegion;
 
   mozilla::UniquePtr<mozilla::VibrancyManager> mVibrancyManager;
   RefPtr<mozilla::SwipeTracker> mSwipeTracker;
   mozilla::UniquePtr<mozilla::SwipeEventQueue> mSwipeEventQueue;
 
-  // Only used for drawRect-based painting in popups.
-  RefPtr<mozilla::gfx::DrawTarget> mBackingSurface;
+  RefPtr<mozilla::CancelableRunnable> mUnsuspendAsyncCATransactionsRunnable;
+
+  // The widget's opaque region. Written on the main thread, read on any thread.
+  mozilla::DataMutex<mozilla::LayoutDeviceIntRegion> mOpaqueRegion;
 
   // This flag is only used when APZ is off. It indicates that the current pan
   // gesture was processed as a swipe. Sometimes the swipe animation can finish
@@ -660,8 +603,6 @@ class nsChildView final : public nsBaseWidget {
 
   static uint32_t sLastInputEventCount;
 
-  void ReleaseTitlebarCGContext();
-
   // This is used by SynthesizeNativeTouchPoint to maintain state between
   // multiple synthesized points
   mozilla::UniquePtr<mozilla::MultiTouchInput> mSynthesizedTouchInput;
